
#include "Arduino.h"
volatile uint16_t icr_1, ocra_1, icr_11, ocra_11, icr_4;
volatile int16_t input=0;
uint32_t period1, period4, period3;
uint16_t diviseur[6] = {0,1,8,64,256,1024};
uint8_t d1,d4,d3;
#define NMOD 100
volatile uint16_t icr_41[NMOD];
uint16_t imod;
float Th;
float Go;

float fo = 1000; // fréquence de référence
float Ko = 100; // gain de l'OCT (en rad/V/s)
float Delta_f = 10.0; // amplitude de la modulation de fréquence
float F = 1.0; // Fréquence de la modulation


void setup() {
  Serial.begin(115200);
DDRB |= 1 << PORTB5; // OC1A (timer 1) configurée en sortie (D11)
//DDRE |= 1 << PORTE3; // OC3A (timer 3) configurée en sortie (D5)
DDRH |= 1 << PORTH3; // OC4A (Timer 4) configurée en sortie (D6)

cli(); // désactivation des interruptions
// Configuration de l'OCT
// Oscillation générée avec Timer1 (sortie D11)
TCCR1A = 0;
TCCR1A |= (1 << COM1A1) ;// clear OC1A on compare match when up-counting, reset OC1A on compare match when down-counting
TCCR1B = 1 << WGM13; // phase and frequency correct pwm mode, top = ICR1
period1 = 1/fo*1e6; // période en microsecondes
d1 = 1;
icr_1 = (F_CPU/1000000*period1/2/diviseur[d1]); // valeur maximale du compteur, pour définir la période
Th = 1.0/(F_CPU/1000000/diviseur[d1]); // période de l'horloge (en us)
Go = 5*Ko/(2*PI*1024)*period1*1e-6;
Serial.print("Go = "); Serial.println(Go,10);
ocra_1 = icr_1/2; // rapport cyclique = 1/2
ICR1 = icr_1;
OCR1A = ocra_1;
TIMSK1 = 1 << TOIE1; // overflow interrupt enable
TCCR1B |= d1; // déclenchement du compteur avec choix du diviseur d'horloge

// Timer 4 pour générer le signal de test de la PLL
TCCR4A = 0;
TCCR4A |= (1 << COM4A1) ;
TCCR4B = 1 << WGM43; 
d4 = 1;
period4 = period1;
Th = 1.0/(F_CPU/1000000/diviseur[d4]); // période de l'horloge (en us)
icr_4 = (F_CPU/1000000*period4/2/diviseur[d4]);
for (uint16_t i=0; i<NMOD; i++) icr_41[i] = 1.0/(1.0/icr_4+2*Th*1e-6*Delta_f*sin(2*PI*i/NMOD));
imod = 0;
ICR4 = icr_4;
OCR4A = icr_4/2;
TCCR4B |= d4; 


//Timer 3 pour générer le signal de modulation de fréquence (pour tester la PLL)
TCCR3A = 0;
TCCR3A |= (1 << COM3A1) ;
TCCR3B = 1 << WGM13; 
d3 = 3;
period3 = (1e6/F)/NMOD; // période du Timer3 pour la modulation (en uS)
ICR3 = (F_CPU/1000000*period3/2/diviseur[d3]);
OCR3A = ICR3 / 2;
TIMSK3 = 1 << TOIE3; // overflow interrupt enable
TCCR3B |= d3;

// configuration de l'ADC
ADCSRA = 0;
ADCSRA |= (1 << ADEN); // enable ADC
ADCSRA |= 0 ; // prescaler
ADCSRB = 0;
ADMUX = 0b01000000 | 0; // REFS0 et ADC0
ADCSRA |= 0b01000000; // start ADC
sei();// activation des interruptions
}


// Changement de la fréquence de l'OCT à chaque cycle
ISR(TIMER1_OVF_vect) { // Timer 1 Overflow interrupt
  input = ADCL | (ADCH <<8);
  icr_11 = 1.0*icr_1/(1+Go*(input*1.0-512.0));
  ocra_11 = icr_11/2;
  ICR1 = icr_11;
  OCR1A = ocra_11;
  ADCSRA |= 0b01000000; // start ADC
}

// modulation de fréquence de Timer4
ISR(TIMER3_OVF_vect) {
   imod += 1;
   if (imod==NMOD) imod=0;
   ICR4 = icr_41[imod];
   OCR4A = ICR4/2;
}

void loop() {
  delay(1000);
  float f = 16e6/(icr_11*2*diviseur[d1]);
  Serial.print(", "); Serial.print(f); Serial.println(" Hz"); 
}	   
    	   